/*
 * Copyright (c) 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.cometd.examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.cometd.bayeux.Channel;
import org.cometd.bayeux.Message;
import org.cometd.bayeux.client.ClientSessionChannel;
import org.cometd.client.BayeuxClient;
import org.cometd.client.transport.LongPollingTransport;

/**
 *
 */
public class SlideShowClient {

    public static void main(String[] args) throws IOException {
        SlideShowClient client = new SlideShowClient();
        client.run();
    }
    private volatile BayeuxClient client;
    private final SlideShowListener slideshowListener = new SlideShowListener();
    
    private void run() throws IOException {
        BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
        
        String defaultURL = "http://localhost:8085/slideshow/cometd";
        
        String url = defaultURL;
        
        client = new BayeuxClient(url, LongPollingTransport.create(null));
        client.getChannel(Channel.META_HANDSHAKE).addListener(new InitializerListener());
        client.getChannel(Channel.META_CONNECT).addListener(new ConnectionListener());
        
        client.handshake();
        boolean success = client.waitFor(1000, BayeuxClient.State.CONNECTED);
        if (!success) {
            System.err.printf("Could not handshake with server at %s%n", url);
            return;
        }
        
        // wait for 6 seconds
        try {
            Thread.sleep(6000);
        } catch (InterruptedException ex) {
            Logger.getLogger(SlideShowClient.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        // set the show to slide number 4 (for no really good reason)
        Map<String, Object> data = new HashMap<String, Object>();
        data.put("uniqueId", "serverSide623736");
        data.put("slideNumber", 4);
        client.getChannel("/slide/show").publish(data);
        

        
                // wait for 30 seconds
        try {
            Thread.sleep(30000);
        } catch (InterruptedException ex) {
            Logger.getLogger(SlideShowClient.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        // set the show to slide number 5 (for no really good reason)
        data = new HashMap<String, Object>();
        data.put("uniqueId", "serverSide 6237366");
        data.put("slideNumber", 5);
        client.getChannel("/slide/show").publish(data);

    }
    
    private void initialize() {
        client.batch(new Runnable() {
            public void run() {
                ClientSessionChannel slideshowChannel = client.getChannel("/slide/show");
                slideshowChannel.unsubscribe(slideshowListener);
                slideshowChannel.subscribe(slideshowListener);
            }
        });
    }
    
    private void connectionEstablished() {
        System.err.printf("system: Connection to Server Opened%n");
    }
    
    private void connectionClosed() {
        System.err.printf("system: Connection to Server Closed%n");
    }
    
    private void connectionBroken() {
        System.err.printf("system: Connection to Server Broken%n");
    }
    
    private class InitializerListener implements ClientSessionChannel.MessageListener {

        public void onMessage(ClientSessionChannel channel, Message message) {
            if (message.isSuccessful()) {
                initialize();
            }
        }
    }
    
    private class ConnectionListener implements ClientSessionChannel.MessageListener {

        private boolean wasConnected;
        private boolean connected;
        
        public void onMessage(ClientSessionChannel channel, Message message) {
            if (client.isDisconnected()) {
                connected = false;
                connectionClosed();
                return;
            }
            
            wasConnected = connected;
            connected = message.isSuccessful();
            if (!wasConnected && connected) {
                connectionEstablished();
            } else if (wasConnected && !connected) {
                connectionBroken();
            }
        }
    }
    
    private class SlideShowListener implements ClientSessionChannel.MessageListener {

        public void onMessage(ClientSessionChannel channel, Message message) {
            Map<String, Object> data = message.getDataAsMap();
            Object uniqueId = data.get("uniqueId");
            Object slideNumber = data.get("slideNumber");
            System.out.println("Received Slide Selection Message: slidenumber: " 
                              + slideNumber + " (Unique Id " + uniqueId + ")");
        }
    }
}
